function [res] = ConfigureSystemBC()
% Configure system model of Balancing Cube with N=6 agents.
%
% --
% Supplementary material (simulation examples) to:
%   Distributed Event-based State Estimation (technical report)
%   Event-based State Estimation: An Emulation-based Approach (journal)
%
% Copyright 2015-17 Max Planck Society. All rights reserved.
% 
% Sebastian Trimpe
% Max Planck Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [31.03.15, ST]    first version
% [20.04.15, ST]    change configuration of input and sensor dims (for
%                   greater flexibility)
% [03.08.15, ST]    some clean up


%% BC model 
% Full model:
%   States: [modPos(1:6), modVel(1:6), cubePos, cubeVel]
%
% From technical report:
% Trimpe/D'Andrea, "Numerical models and controller design parameters for
% the Balancing Cube"
% TSS model, edge-balancing (Sec. 3.1)
Ts = 1/100;
A31 = [...
   1.0043e-05  -2.0085e-05  -1.0043e-05  -7.0769e-06   1.0008e-05   7.0769e-06
   2.0087e-03  -4.0173e-03  -2.0087e-03  -1.4155e-03   2.0018e-03   1.4155e-03 ];
A32 = [ ...
   3.7758e-05  -1.9289e-04  -3.7759e-05  -2.1818e-04   1.0951e-04   2.1818e-04
   3.7770e-03  -1.9295e-02  -3.7770e-03  -2.1825e-02   1.0955e-02   2.1825e-02 ];
A33 = [...
   1.0005e+00   1.0002e-02
   9.2624e-02   1.0005e+00 ];
B3 = [...
  -3.7725e-05   1.9282e-04   3.7725e-05   2.1816e-04  -1.0948e-04  -2.1816e-04
  -3.7669e-03   1.9275e-02   3.7670e-03   2.1818e-02  -1.0945e-02  -2.1818e-02 ];

% Full model (including previous inputs as states)
% States: [modPos(1:6), modVel(1:6), cubePos, cubeVel]
A = [  eye(6), zeros(6), zeros(6,2); 
            zeros(6), zeros(6), zeros(6,2);
            A31, A32, A33 ];
B = [  Ts*eye(6); eye(6); B3 ];

% Measurements
% All agents measure their angle.
% Agents 1..3 measure cube velocity; others don't.
C = [  1 0 0 0 0 0   0 0 0 0 0 0   0 0;
        0 0 0 0 0 0   0 0 0 0 0 0   0 1;
        0 1 0 0 0 0   0 0 0 0 0 0   0 0;
        0 0 0 0 0 0   0 0 0 0 0 0   0 1;
        0 0 1 0 0 0   0 0 0 0 0 0   0 0;
        0 0 0 0 0 0   0 0 0 0 0 0   0 1;
        0 0 0 1 0 0   0 0 0 0 0 0   0 0;
        0 0 0 0 1 0   0 0 0 0 0 0   0 0;
        0 0 0 0 0 1   0 0 0 0 0 0   0 0 ];

% Initial condition
x_init = zeros(length(A),1);


%% Reduced system model
% Use reduced representation exploiting the structure of the above model,
% namely x_{7:12}(k) = u(k-1).  That is, we use the model
%   x(k+1) = A*x(k) + B*u(k) + Bminus*u(k-1)
% 
% Reduced states: [modPos(1:6), cubePos, cubeVel]
Ar = [   eye(6), zeros(6,2); 
        A31, A33 ];
Br = [  Ts*eye(6); B3 ];
Br_minus = [ zeros(6); A32 ];

% Measurements
% Agents 1..3 measure cube velocity; others don't
Cr = C;
Cr(:,7:12) = [];
% Cr = [   1 0 0 0 0 0 0 0;
%         0 0 0 0 0 0 0 1;
%         0 1 0 0 0 0 0 0;
%         0 0 0 0 0 0 0 1;
%         0 0 1 0 0 0 0 0;
%         0 0 0 0 0 0 0 1;
%         0 0 0 1 0 0 0 0;
%         0 0 0 0 1 0 0 0;
%         0 0 0 0 0 1 0 0 ];
    
% x_init, reduced state
x_init_r = x_init([1:6,13:14]);

    
%% Agents and indices
% Number of agents:
numAgents = 6; 

% Measurement indices corresponding to 6 agents.
% Each entry (cell) corresponds to one agent.  For each agent, a cell 
% specifies what measurements belong to that agent.  Indeces are into rows
% of C.
%
% Example:
%   y_dims{2} = {4, 5};     % Agent 2 has 2 sensors: y4 and y5
%   y_dims{4} = {(6:7)'};      % Agent 4 has 1 sensor: y_{6:7}
%
% Indices (e.g. (6:7)') must be column vectors.
y_dims{1} = {1, 2};     % Agent 1: 2 sensors: y1 and y2
y_dims{2} = {3, 4};     % Agent 2: 2 sensors: y3 and y4
y_dims{3} = {5, 6};     % etc.
y_dims{4} = {7};     
y_dims{5} = {8};     
y_dims{6} = {9};     

% Inputs: same convention as for measurements
u_dims{1} = {1};    % Agent 1: 1 actuator: u1
u_dims{2} = {2}; 
u_dims{3} = {3}; 
u_dims{4} = {4}; 
u_dims{5} = {5}; 
u_dims{6} = {6}; 

        
%% Noise
% All noise assumed uniform.
gyroVar = 1.7e-05;  % corresponds the Balancing Cube sensor

% Noise limit; noise is uniform in [-NoiseLim, +NoiseLim].
gyroNoiseLim = sqrt(3*gyroVar); % Sensor noise: rate gyro, []=rad/s
encNoiseLim = (2*pi/4096)/2;    % Sensor noise: angle encoder; encoder resolution: 4096 steps; []=rad
inpNoiseLim = 0.05;             % Process/input noise, []=rad

% Measurement noise
w_lim = [encNoiseLim, gyroNoiseLim, encNoiseLim, gyroNoiseLim, ...
    encNoiseLim, gyroNoiseLim, encNoiseLim, encNoiseLim, encNoiseLim]';

% Input process noise
vInput_lim = inpNoiseLim*ones(6,1);


%% General disturbance
% General disturbance signal d(k) acting on the state as E*d(k)
dist.enabled = true;
dist.E = B;     % input disturbance

% Disturbance signal
ind_u = 2;      % which input channel?
amp = 1.5;      % which amplitude?
sig = zeros(20000,6);
sig(2000:2020,ind_u) = amp;  % dist. active from 20 sec till 20.1 sec
dist.d_signal = sig;


%% Output results
res.A = A;
res.B = B;
res.C = C;
res.Ts = Ts;
res.x_init = x_init;
res.numAgents = numAgents;
res.n_x = length(A);
res.n_u = size(B,2);
res.n_y = size(C,1);
res.u_dims = u_dims;
res.y_dims = y_dims;
res.w_lim = w_lim;
res.vInput_lim = vInput_lim;
res.dist = dist;

% Estimator model: this is the model that the state estimator uses.
% This may be the model A,B,C above or a different one ("reduced order
% model")
res.est.flagReducedModel = true;
res.est.A = Ar;
res.est.B = Br;
res.est.Bminus = Br_minus;
res.est.C = Cr;
res.est.n_x = length(Ar);   % state dimension (reduced)
res.est.n_u = res.n_u;
res.est.x_init = x_init_r;
